/*
   _===_ _           _                  ______ _     _             _ _ _
  |  ___| |         | |                 | ___ (_)   | |           (_) | |
  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|

  Essais du Synthétiseur RF Analog Devices ADF4351
  Code Source pour Arduino UNO v3, compilé sur Arduino IDE
  Électro-Bidouilleur, Avril 2021. http://bidouilleur.ca
  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr

  Inspiré du code, et utilisant la librairie ADF4351 de: 
  Licence MIT: Copyright (c) 2018 David Fannin: https://github.com/dfannin/adf4351 ,
  laquelle utilise la librairie BigNumber de:
  Licence MIT: Copyright (c) 2017-2018 S. Downey:  https://github.com/dfannin/adf4351/tree/master/lib/BigNumber
  The above copyright notices and this permission notice shall be included in all copies or substantial portions of the Software.

  Brochage de l'ADF4351 sur l'Arduino UNO: 
    LE (SS) - Arduino D10
    DAT     - Arduino D11
    CLK     - Arduino D13
   Note: Des atténuateurs résistifs sont requis sur ces 3 signaux car les entrées de l'ADF4351 
         ne sont pas compatibles TTL 5V:

                      2K
   Signal Arduino ---/\/\/\---+--- Signal ADF4351
                             |
                             |
                             /
                             \
                        3.3K /
                             \
                             /
                             |
                             |
                            GND
 */

#include <adf4351.h>        // Librairie ADF4351 de David Fannin. Utilise aussi la librairie "BigNumber"
#include <Arduino.h>

#define PIN_SS 10           // Assignation de la broche slave select de SPI

ADF4351  vfo(PIN_SS, SPI_MODE0, 100000UL , MSBFIRST) ; // Assigner les bon paramètres au constructeur.

// Fonction D'initialisation, style Arduino
void setup()
{
  Serial.begin(9600) ;      // Configurer le port série
  Wire.begin() ;            // Activer la connexion SPI

  // Configurer la puce (la plupart de ces valeurs sont celles par défaut)
  vfo.pwrlevel = 0;         // amplitude de sortie à -4 dBm (valeurs 0, 1, 2, 3 correspondent à -4dBm, -1dBm, +2 dBm, +5dBm)
  vfo.RD2refdouble = 0 ;    // ref doubleur off
  vfo.RD1Rdiv2 = 0 ;        // ref diviseur off
  vfo.ClkDiv = 150 ;
  vfo.BandSelClock = 80 ;
  vfo.RCounter = 1 ;        // Compteur R à 1 (pas de division)
  vfo.ChanStep = steps[2] ; // Sauts de 10 kHz

  // Configurer la fréquence de référence 
  if ( vfo.setrf(25000000UL) ==  0 ) Serial.println("Fréquence de référence: 25 MHz") ;  // Si fait avec succès, afficher la fréquence 
  else Serial.println("Erreur de setrf() (fréquence de référence)") ;                    // Sinon, afficher un message d'erreur
  
  // Initialiser la puce
  vfo.init() ;              // Initialiser la puce
  vfo.enable() ;            // Activer la sortie RF

  Serial.print("Entrez la  Fréquence (Hz): ") ;
}

// Fonction de boucle principale, style Arduino
void loop()
{
static char freq_string[11];                  // La chaîne de caractères (maximum 11 de longueur)
static unsigned char i;                       // Le compteur de position des caractères
 
  if (Serial.available() > 0)                 // charactère reçu?
  {    
    freq_string[i] = Serial.read();           // Lire le charactère et le transférer dans la chaîne de caractères à la position courante
    if (freq_string[i] == 10)                 // Est-ce un caractère de fin de ligne (LF)?
    { // oui 
      freq_string[i] = NULL;                  // Remplacer par Caractere NULL dans la chaîne de caractères
     if ( vfo.setf(strtoul(freq_string,NULL,0)) == 0 )      // Envoyer la nouvelle fréquence à l'ADF5341
      {                                       // Si fait avec succès, afficher les différents paramètres utilisés
        Serial.println("");
        Serial.println("setf() réussi.");
        Serial.print("VFO CFREQ: ");
        Serial.println(vfo.cfreq) ;
        Serial.print("PLL INT: ");
        Serial.println(vfo.N_Int);
        Serial.print("PLL FRAC: ");
        Serial.println(vfo.Frac);
        Serial.print("PLL MOD: ");
        Serial.println(vfo.Mod);
        Serial.print("PLL PFD: ");
        Serial.println(vfo.PFDFreq);
        Serial.print("PLL output divider: ");
        Serial.println(vfo.outdiv);
        Serial.print("PLL prescaler: ");
        Serial.println(vfo.Prescaler);
        Serial.println("");
      } 
      else Serial.println("Erreur de setf()") ;     // Sinon, afficher un message d'erreur.
      Serial.print("Entrez la Fréquence (Hz): ") ;  // Demander d'entrer une nouvelle fréquence
      i = 0;        // Ré-initialiser le compteur de position des caractères
    }
    else i++;       // Incrémenter le compteur de position des caractères
  }
}
